#include "config.h"
#include "lib.h"
#include <stdio.h>
#include <unistd.h>
#include <sys/time.h>
#include <string.h>
#include <stdlib.h>

char window[128];
extern int sock_fd;

void parse_line()
{
	char line[1024];
	int n;
	struct packet *p;

	n = read(0, line, 1000);
	if (n <= 0)
		exit(0);
	if (line[n - 1] == '\n')
		n--;
	line[n] = 0;
	if (strncasecmp(line, "/window ", 8) == 0) {
		strncpy(window, line + 8, 100);
	} else {
		p = packet_printf("%s%s", window, line);
		packet_send(sock_fd, CMD_INPUT, p);
		free(p);
	}
}

void print(const char *window, const char *s)
{
	printf("%s: ", window ? window : "status");
	while (*s)
		if (*s == 27 && s[1])
			s += 2;
		else
			putchar(*s++);
	putchar('\n');
}

void kill_window(const char *name)
{
	printf("window '%s' killed\n", name);
}

void add_window(const char *name)
{
	printf("window '%s' added\n", name);
}

void parse_packet()
{
	int cmd;
	struct packet *p;
	char *win, *arg;
	
	p = packet_recv(sock_fd, &cmd);
	if (p == 0) {
		fprintf(stderr, "connection lost ...\n");
		exit(1);
	}
	
	switch (cmd) {
	case CMD_PRINT:
		packet_scanf(p, "%s%s", &win, &arg);
		print(win, arg);
		free(win);
		free(arg);
		break;
	case CMD_EXIT:
		packet_scanf(p, "%s", &arg);
		print(0, arg);
		free(arg);
		exit(0);
		break;
	case CMD_NEW_WINDOW:
		packet_scanf(p, "%s", &win);
		add_window(win);
		free(win);
		break;
	case CMD_KILL_WINDOW:
		packet_scanf(p, "%s", &win);
		kill_window(win);
		free(win);
		break;
	}
	free(p);
}

extern char *xversion, *xdate;
void iface_init(int retach);

int main()
{
	fd_set xx;
	
	printf("schizophrenia %s [%s]\n",xversion,xdate);
	printf("y2k by malekith@topnet.pl\n");
	printf("connecting...\n");
	
	iface_init(1);
	
	for (;;) {
		FD_ZERO(&xx);
		FD_SET(0, &xx);
		FD_SET(sock_fd, &xx);
		select(sock_fd + 1, &xx, 0, 0, 0);
		if (FD_ISSET(sock_fd, &xx))
			parse_packet();
		if (FD_ISSET(0, &xx))
			parse_line();
	}
	
	return 0;
}

