#include<stdlib.h>
#include<string.h>
#include<stdio.h>

/* message :
byte r[4];	random
byte message[];
message encrypted with key ^ r (cyclic)
message[0] == '-' && message[strlen(message)-1] == '-'; to allow
	easy reconizing whatever we have good key 
*/

static char enc[] = 
"ucrgtglytpkutrgebertnpegusipcruidpdinissglivanrpffsigusukaheyccs"
"ebpsfagratconainpeybbstoocibnfgattaetabnuditiflmamirjehyfefocaht"
"nslcenichomplietbcreiopuhadaorserfruzarcchcefrubtuesdrfltlftalcl"
"cpazumsmvathpaeaagrdwiaqbloptionutnteunuoumlilacnvobbrpiscefmoof"
"nhrancrockrsngrrmarqsfafmeimunejklquhuepytelomslndlgysdsgnvedkso"
"otjaaraohidoeicuurueskkbbugzkeldoakwdujobomiosleadzideprrneyolmm"
"giezwwrvabloymnjdcviaictlvvoauedulemtcuaujigoqupowcievavpcoderll"
"mbshtszewaptfiltufokwtccstrisyizsppolhsrisiqmfphoobtdmideelatemu";

void rc4(unsigned char *key, int key_len,
	 unsigned char *data, int data_len)
{
	unsigned char s[256], i, j;
	int k, tmp;

	i = 255;
	do
		s[i] = i;
	while (i--);
	
	i = j = k = 0;
	do {
		j += s[i] + key[k];
		k = (k + 1) % key_len;
		tmp = s[i];
		s[i] = s[j];
		s[j] = tmp;
	} while (++i);
	
	i = j = 0;

	while (data_len--) {
		s[i] = s[j += (tmp = s[++i])];
		s[j] = tmp;
		*data++ ^= s[(s[i] + s[j]) & 0xff];
	}
}

int decode(const char *in, char *out, int *ol)
{
	int b1, b2, i;
	char *p;

	p = out;
	for (;;) {
		while (*in && strchr(" ,.'", *in))
			in++;
		if (*in == 0)
			break;
		b1 = *in++ | 0x20;
		while (*in && strchr(" ,.'", *in))
			in++;
		if (*in == 0)
			return -1;
		b2 = *in++ | 0x20;
		for (i = 0; i < 256; i++)
			if (enc[i * 2] == b1 && enc[i * 2 + 1] == b2)
				break;
		if (i == 256)
			return -1;
		*p++ = i;
	}
	if (p == out)
		return -1;
	*ol = p - out;
	return 0;
}

struct ctx {
	const char *in, *t;
	char buf[4];
	int left;
};

static int getbyte(struct ctx *o)
{
	if (o->t && *o->t)
		return *o->t++;
	if (o->left == 0)
		return 0;
	memcpy(o->buf, enc + ((unsigned char) *o->in++) * 2, 2);
	o->buf[2] = 0;
	o->t = o->buf;
	o->left--;
	return *o->t++;
}

char *encode(const char *dat, int len, int b)
{
	struct ctx c;
	int word, apo, part, sent, x;
	char buf[4096], *p;
	
	c.in = dat;
	c.t = 0;
	c.left = len;
	
	p = buf;
	if (b) {
		word = rand() % 8 + 1;
		apo = 0;
		part = rand() % 4 + 1;
		sent = rand() % 10 + 1;
	} else {
		word = -1;
		apo = -1;
		part = -1;
		sent = -1;
	}
	
	while ((x = getbyte(&c))) {
		if (word-- == 0) {
			word = rand() % 8 + 1;
			apo = rand() % 10 + 1;
			if (sent-- == 0) {
				sent = rand() % 10 + 1;
				part = rand() % 4 + 1;
				*p++ = '.';
				*p++ = ' ';
				*p++ = x & ~0x20;
			} else if (part-- == 0) {
				part = rand() % 4 + 1;
				*p++ = ',';
				*p++ = ' ';
				*p++ = rand() % 10 ? x : x & ~0x20;
			} else {
				*p++ = ' ';
				*p++ = rand() % 10 ? x : x & ~0x20;
			}
		} else if (p == buf)
			*p++ = x & ~0x20;
		else if (apo-- == 0) {
			*p++ = x;
			*p++ = '\'';
		} else
			*p++ = x;
	}
	*p++ = '.';
	*p = 0;
	return strdup(buf);
}

char *cipher(const char *message, const char *key, int b)
{
	char *buf, *p, *k;
	int n, l;

	l = strlen(message);
	buf = malloc(4 + 1 + l + 1);
	strcpy(buf + 4 + 1, message);
	buf[4] = '-';
	buf[4 + 1 + l] = '-';
	*((int *) buf) = rand();
	p = k = strdup(key);
	n = 0;
	while (*p) {
		*p++ ^= buf[n++];
		n &= 0x3;
	}
	rc4(k, strlen(key), buf + 4, l + 2);
	p = encode(buf, l + 6, b);
	free(buf);
	return p;
}

char *decipher(const char *message, const char *keylist)
{
	int l, n, no;
	char *buf, *r = 0, *k, *p, *e, *s;
	
	buf = malloc(strlen(message) / 2 + 10);
	if (decode(message, buf, &l) || l < 7) {
		free(buf);
		return 0;
	}
	buf[l] = 0;
	k = p = strdup(keylist);
	no = 0;
	while (p && *p) {
		no++;
		e = strchr(p, '\n');
		if (e)
			*e++ = 0;
		s = p;
		n = 0;
		while (*s) {
			*s++ ^= buf[n++];
			n &= 3;
		}
		rc4(p, s - p, buf + 4, l - 4);
		if (buf[4] == '-' && buf[l - 1] == '-') {
			if (r)
				r = realloc(r, strlen(r) + 10 + l - 6);
			else {
				r = malloc(10 + l - 6);
				r[0] = 0;
			}
			sprintf(strchr(r, 0), "%d: %s", no, buf + 5);
			strchr(r, 0)[-1] = 0;
		}
		rc4(p, s - p, buf + 4, l - 4);	/*undo */
		p = e;
	}
	return r;
}

int main()
{
	char inbuf[1000], *x,*x2;
	int i;
/*
	while (fgets(inbuf, 1000, stdin)) {
		x = cipher(inbuf, "eh ziuta", 0);
		puts(x);
		x = decipher(x, "holalal\neh ziuta\ndupa\n1234");
		puts(x ? : "x==0");
	}*/
	strcpy(inbuf,"hehehehdehheekakakkakaaaakakak");
	for(i=0;i<1000;i++){
	x=cipher(inbuf, "eh ziuta", 1);
	x2=decipher(x, "holalal\neh ziuta\ndupa\n1234\nhahahahaha\naaaaaaa");
	free(x);free(x2);}
}
