/* include gnu.gpl, (*) 2000 Micha 'malekith' Moskal <malekith@topnet.pl>. 
 * $Id: window.c,v 1.8 2000/08/26 01:03:48 malekith Exp $ */

#include "bot.h"
#include "var.h"

#include <stdio.h>

struct line {
	struct line *next, *prev;
	int no;
	char data[0];
};

struct window {
	struct window *next;
	char *title;
	int line;
	
	struct line *first, *last;
};

static struct window *win_head;

static struct line *compile_line(char **pp)
{
	int len;
	struct line *r;

	len = line_count_len(*pp);
	r = xmalloc(sizeof(struct line) + len + 1);
	r->next = r->prev = 0;
	
	*pp = line_do_compile(*pp, r->data);
	
	return r;
}

static void win_flush(struct window *w, struct line *l)
{
	xsend(CMD_PRINT, "%s%s", w->title ? w->title : "status", l->data);
}

static void win_flushm(struct window *w, int start, int n)
{
	struct line *l;
	
	if (start < 1)
		start = w->line - n;
	if (start < 1)
		start = 1;

	for (l = w->first; l && l->no < start; l = l->next)
		/* nothing */;
	for (; l && n--; l = l->next)
		win_flush(w, l);
}

void win_sync(int n)
{
	struct window *w, *p;

	xsend(CMD_FREEZE, "%d", 1);

	w = win_head;
	while (w && w->next)
		w = w->next;

	for (;;) {
		if (w->title)
			xsend(CMD_NEW_WINDOW, "%s", w->title);
		
		win_flushm(w, 0, n);

		if (w == win_head)
			break;
		for (p = win_head; p && p->next != w; p = p->next)
			/* nothing */;
		if ((w = p) == 0)
			break;
	}
	xsend(CMD_FREEZE, "%d", 0);
}

static void xlog(const char *l, const char *win)
{
	static FILE *log;
	static char *logfile;

	if (logfile == 0 || strcmp(logfile, sv("logfile")->var_string)) {
		if (log)
			fclose(log);
		xfree(logfile);
		logfile = xstrdup(sv("logfile")->var_string);
		log = *logfile ? fopen(logfile, "a") : 0;
	}
	
	if (log == 0)
		return;
		
	fprintf(log, "[%-8s]: ", win ? win : "STATUS");
	for (; *l; l++)
		if (*l == 27)
			l++;
		else
			fputc(*l, log);
	fputc('\n', log);
	fflush(log);
}

static void win_add_text(struct window *w, char *line)
{
	struct line *x;
	
	while (line && *line) {
		x = compile_line(&line);
		if (w->first == 0)
			w->first = x;
		if (w->last)
			w->last->next = x;
		x->prev = w->last;
		w->last = x;
		x->no = ++w->line;
		win_flush(w, x);
		xlog(x->data, w->title);
	}
}

static struct window *win_new(char *title)
{
	struct window *w;
	w = NEW(struct window);
	w->next = 0;
	w->title = title;
	w->first = w->last = 0;
	w->line = 0;
	return w;
}

static struct window *win_find(const char *title)
{
	struct window *w = 0;
	if (title)
		for (w = win_head; w; w = w->next)
			if (w->title && same(w->title, title))
				break;
	if (w == 0)
		for (w = win_head; w; w = w->next)
			if (w->title == 0)
				break;
	return w;
}

static void win_destroy(struct window *w)
{
	struct line *l, *tmp;
	
	for (l = w->first; l; l = tmp) {
		tmp = l->next;
		xfree(l);
	}
	
	xfree(w->title);
	xfree(w);
}

void win_init()
{
	win_head = win_new(0);
}

void win_add(const char *title)
{
	struct window *w;

	w = win_new(xstrdup(title));
	w->next = win_head;
	win_head = w;
	
	xsend(CMD_NEW_WINDOW, "%s", title);
}

void win_kill(const char *title)
{
	struct window *w, *p;
	
	w = win_find(title);
	if (w->title == 0)
		return;
		
	/* unlink sucker ;) */
	if (w == win_head) {
		win_head = win_head->next;
	} else {
		for (p = win_head; p->next; p = p->next)
			if (p->next == w)
				break;
		if (p->next)
			p->next = p->next->next;
	}
	
	xsend(CMD_KILL_WINDOW, "%s", title);
	
	win_destroy(w);
}

void prt(const char *win, const char *blah)
{
	char *c;
	c = xstrdup(blah);
	win_add_text(win_find(win), c);
	xfree(c);
}

void prtf(const char *win, const char *fmt, ...)
{
	va_list ap;
	char *p;

	va_start(ap, fmt);
	p = vxf(fmt, ap);
	prt(win, p);
	xfree(p);
	va_end(ap);
}
