/* include gnu.gpl, (*) 2000 Micha 'malekith' Moskal <malekith@topnet.pl>. 
 * $Id: alloc.c,v 1.1 2000/08/04 21:28:14 malekith Exp $ */

#define _GNU_SOURCE

#include "config.h"
#include "lib.h"

#include <stdlib.h>
#include <stdio.h>
#include <string.h>

struct mem_node {
	void *ptr;
	int len;
	const char *file;
	int line;
	u32 flags;
};

static struct mem_node *nodes;
static int n_nodes;

int alloc_print;

void xinfo(u32 flag_set, u32 flag_mask)
{
	int i, sum = 0, s2 = 0;
	
	if (n_nodes == 0)
		return;
		
	printf("************************* MEMORY USAGE STATS:\n");
	
	for (i = 0; i < n_nodes; i++)
		if (nodes[i].ptr && (nodes[i].flags & flag_mask) == 0) {
			printf("%p %5d bytes %s : %d\n", nodes[i].ptr,
			       nodes[i].len, nodes[i].file, nodes[i].line);
			sum += nodes[i].len;
			s2 += (nodes[i].len & 0xfffffffa) + 8;
			nodes[i].flags |= flag_set;
		}
		
	printf("TOTAL: %d bytes (%d)\n\n", sum, s2);
	(void)flag_set;
	(void)flag_mask;
}


static void add_node(void *p, int n, const char *f, int l)
{
	int i;
	
	for (i = 0; i < n_nodes; i++)
		if (nodes[i].ptr == 0) 
			break;
	if (i == n_nodes) {
		nodes = realloc(nodes, 
				(n_nodes += 100) * sizeof(struct mem_node));
		for (; i < n_nodes; i++) 
			nodes[i].ptr = 0;
		i--;
	}
		
	nodes[i].ptr = p;
	nodes[i].len = n;
	nodes[i].file = f;
	nodes[i].line = l;
	nodes[i].flags = 0;
}

static void delete_node(void *p)
{
	int i;

	for (i = 0; i < n_nodes; i++)
		if (nodes[i].ptr == p)
			break;
	if (i == n_nodes)
		abort();
	nodes[i].ptr = 0;
}

void *do_xmalloc(int n, const char *f, int l)
{
	void *p;
	
	p = malloc(n);

	if (alloc_print) 
		printf("malloc %8d = %p [%s:%d]\n", n, p, f, l);

	add_node(p, n, f, l);
	
	return p;
}

void *do_xrealloc(void *p, int n, const char *f, int l)
{
	void *x;
	x = realloc(p, n);

	if (alloc_print) 
		printf("xrealloc (%p,%d) = %p [%s:%d]\n", p, n, x, f, l);

	delete_node(p);
	add_node(x, n, f, l);
	
	return x;
}

void do_xfree(void *p, const char *f, int l)
{
	if (alloc_print)
		printf("free   %8s   %p [%s:%d]\n", "", p, f, l);

	delete_node(p);
	free(p);
}

char *do_xstrdup(const char *p, const char *f, int l)
{
	char *r;
	
	r = do_xmalloc(strlen(p) + 1, f, l);
	strcpy(r, p);
	if (alloc_print) 
		printf("strdup %s [%s:%d]\n", p, f, l);
	
	return r;
}

char *do_xstrndup(const char *src, int n, const char *f, int l)
{
	char *r;

	r = (char*)do_xmalloc(n + 1, f, l);
	r[n] = 0;
	strncpy(r, src, n);
	return r;
}

char *do_vxf(const char *f, int l, const char *fmt, va_list ap)
{
	char *p;


	vasprintf(&p, fmt, ap);
	
	add_node(p, strlen(p) + 1, f, l);
	
	return p;
}

char *do_xf(const char *f, int l, const char *fmt, ...)
{
	va_list ap;
	char *r;

	va_start(ap, fmt);
	r = do_vxf(f, l, fmt, ap);
	va_end(ap);
	return r;
}

#undef xmalloc
#undef xfree
#undef xrealloc
#undef xstrdup
#undef xstrndup
#undef xf
#undef vxf

void *xmalloc(int n)
{
	void *p;
	
	p = malloc(n);
	
	return p;
}

void *xrealloc(void *p, int n)
{
	void *x;
	
	x = realloc(p, n);

	return x;
}

void xfree(void *p)
{
	free(p);
}

char *xstrdup(const char *p)
{
	char *r;
	
	r = xmalloc(strlen(p) + 1);
	strcpy(r, p);
	
	return r;
}

char *xstrndup(const char *src, int n)
{
	char *r;

	r = (char*)xmalloc(n + 1);
	r[n] = 0;
	strncpy(r, src, n);
	return r;
}

char *vxf(const char *fmt, va_list ap)
{
	char *p;


	vasprintf(&p, fmt, ap);
	
	return p;
}

char *xf(const char *fmt, ...)
{
	va_list ap;
	char *r;

	va_start(ap, fmt);
	r = vxf(fmt, ap);
	va_end(ap);
	return r;
}

