#include "config.h"
#include "lib.h"
#include <unistd.h>
#include <dirent.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <fcntl.h>
#include <stdio.h>
#include <string.h>

int sock_fd;

static void spawn_bot()
{
	int pid;
	pid = fork();
	if (pid == 0) {
		char *p, *e, *n;
		p = xstrdup(getenv("PATH"));
		
		/* first try home ... */
		n = xf("%s/schizobot", home_dir);
		execl(n, n, 0);
		xfree(n);
		
		/* and then $PATH */
		while (p && *p) {
			e = strchr(p, ':');
			if (e)
				*e++ = 0;
			n = xf("%s/schizobot", p);
			execl(n, n, 0);
			p = e;
		}
		perror("couldn't spawn bot");
		/* oops */
		exit(1);
	}
	sleep(1);
}

static int try_auth(int sock, char *pass)
{
	char buf[128], *x;
	
	read(sock, buf, 32);
	strcpy(buf + 32, pass);
	
	x = count_md5(buf);
	write(sock, x, 32);
	xfree(x);
	
	if (read(sock, buf, 2) != 2) {
		close(sock);
		return -1;
	}
	
	sock_fd = sock;
	return 0;
}

static int try_conn(int port, char *pass)
{
	struct sockaddr_in sin;
	int sock;

	sin.sin_family = AF_INET;
	sin.sin_port = htons(port);
	sin.sin_addr.s_addr = htonl(0x7f000001);
	
	sock = socket(AF_INET, SOCK_STREAM, 0);
	if (sock == -1) {
		perror("socket");
		exit(1);
	}
	
	if (connect(sock, (struct sockaddr *) &sin, sizeof(sin))) {
		perror("connect");
		return -1;
	} else
		return try_auth(sock, pass);
}

static int try_connect(const char *fn)
{
	int fd;
	struct packet *p;
	int cmd;
	int port;
	char *pass = 0;

	fd = open(fn, O_RDONLY, 0);
	if (fd == -1)
		return -1;
		
	p = packet_recv(fd, &cmd);
	close(fd);
	
	if (p == 0)
		return -1;
		
	packet_scanf(p, "%d%s", &port, &pass);
	xfree(p);
	
	if (pass == 0)
		return -1;
		
	return try_conn(port, pass);
}

static void make_conn()
{
	DIR *d;
	struct dirent *e;

	chdir(home_dir);
	chdir("screens");
	d = opendir(".");
	while ((e = readdir(d)))
		if (try_connect(e->d_name) == 0)
			break;
	closedir(d);
}

static u32 bool1;
static char colors[100];

static void init_conf()
{
	bool1 = (1 << B_COPY_STATUS);
}


void iface_init(int retach)
{
	char dir[1024];
	
	getcwd(dir, 1000);
	
	init_conf();
	init_home_dir();
	if (!retach)
		spawn_bot();
	make_conn();
	if (sock_fd == 0)
		exit(1);
	chdir(dir);
}

void handle_conf(int cmd, struct packet *p)
{
	char *arg;
	
	switch (cmd) {
	case CMD_CONF_COLORS:
		packet_scanf(p, "%s", &arg);
		strncpy(colors, arg, 99);
		xfree(arg);
		break;
	case CMD_CONF_BOOL1:
		packet_scanf(p, "%d", &bool1);
		break;
	default:	
		abort();
	}
}

int get_color(int no)
{
	if (no < 0 || no > (int)strlen(colors))
		return COLOR(LIGHTGRAY, BLACK);
	return colors[no];
}

int get_bool(int no)
{
	return (bool1 & (1 << no)) != 0;
}
