/* include gnu.gpl, (*) 2000 Micha 'malekith' Moskal <malekith@topnet.pl>. 
 * $Id: pexpr.c,v 1.11 2000/08/04 21:28:14 malekith Exp $ */

#include "parser.h"

static token *get_val(CTX * ctx);

byte *bc_get_val(char *s, struct frame *f)
{
	CTX ctx;
	token *t;
	memset(&ctx, 0, sizeof(ctx));

	ctx.p = s;
	ctx.fn = "$";
	ctx.frm = f;
	t = get_val(&ctx);
	return pack(t, 0, 0, 0);
}

static token *pre_op(CTX * ctx)
{
	token *x, *xx;

	switch (ctx->iv) {
		/* pre versions: */
	case 'm':
	case 'p':
		ctx->iv = toupper(ctx->iv);
	case '!':
	case '+':
	case '-':
	case '~':
		/* a normal one arg operator ... */
		x = make_token(ctx, ID_EXPR1, 0, ctx->iv);
		if ((x->left = get_val(ctx)) == 0) {
			kill_token(x);
			x = 0;
		}
		return x;
	case '(':
		x = get_expression(ctx);
		if (x == 0)
			return 0;
		if (ctx->type != T_OP || ctx->iv != ')') {
			poops(ctx, "expression finished wrong (not ')')");
			kill_token(x);
			return 0;
		}
		if (get_token(ctx) == 0) {
			kill_token(x);
			poops(ctx,
			     "unexpected EOF, looking for postexpression");
			return 0;
		}
		if (ctx->type == T_OP && (ctx->iv == 'm' || ctx->iv == 'p')) {
			xx = make_token(ctx, ID_EXPR1, 0, ctx->iv);
			xx->left = x;
			x = xx;
		} else
			ctx->leave_it = 1;
		return x;
	default:
		poops(ctx, "unexpected operator `%s'",op2_name(ctx->iv));
		return 0;
	}
}

static token *post_check(CTX *ctx, token *t)
{
	token *r;
	
	if (get_token(ctx) == 0)
		return t;
	
	if (ctx->type == T_OP)
		switch (ctx->iv) {
		case 'm':
		case 'p':
			r = make_token(ctx, ID_EXPR1, 0, ctx->iv);
			r->left = t;
			t = r;
			break;
		default:
			ctx->leave_it = 1;
			break;
		}

	return t;
}

static token *got_hash(CTX * ctx, char *kw)
{
	token *x;

	x = make_token(ctx, ID_HASH, kw, 0);

	x->left = get_expression(ctx);

	if (ctx->type != T_OP || ctx->iv != ']') {
		poops(ctx, "bad hash expr finishing operator");
		kill_token(x);
		return 0;
	}

	return post_check(ctx, x);
}

static token *got_call(CTX * ctx, char *kw)
{
	token *x, *p;

	x = make_token(ctx, ID_CALL, kw, 0);
	get_token(ctx);
	if (ctx->type == T_OP && ctx->iv == ')')	/* empty () */
		return x;
	ctx->leave_it = 1;

	for (;;) {
		if (link_end(&(x->left), &p, get_expression(ctx)) == 0)
			goto shit;

		if (ctx->type == T_OP) {
			if (ctx->iv == ')')
				break;
			else if (ctx->iv == ',')
				/* nothing */ ;
			else {
				poops(ctx, "bad expr finishing operator");
				goto shit;
			}
		} else {
			poops(ctx, "expr not finished by operator");
			goto shit;
		}
	}
	return x;

      shit:
	kill_token(x);
	return 0;
}

static token *got_word(CTX * ctx)
{
	char *kw;

	if (is_keyword(ctx->sv)) {
		poops(ctx, "unexpected keyword '%s'", ctx->sv);
		return 0;
	}

	kw = ctx->sv;
	ctx->sv = 0;

	if (get_token(ctx) == 0) {
		poops(ctx,
		      "unexpected EOF, looking for postexpression for %s",
		      kw);
		xfree(kw);
		return 0;
	}

	if (ctx->type == T_OP)
		switch (ctx->iv) {
		case '(':
			return got_call(ctx, kw);
		case '[':
			return got_hash(ctx, kw);
		default:
			ctx->leave_it = 1;
			break;
		}

	return post_check(ctx, make_token(ctx, ID_VAR, kw, 0));
}

static token *get_val(CTX * ctx)
{
	if (!get_token(ctx)) {
		poops(ctx, "got unexpct. EOF");
		return 0;
	}

	switch (ctx->type) {
	case T_WORD:
		return got_word(ctx);
	case T_OP:
		return pre_op(ctx);
	case T_INT:
		return make_token(ctx, ID_INT, 0, ctx->iv);
	case T_STRING:
		{
			char *s;
			
			s = ctx->sv;
			ctx->sv = 0;
			return make_token(ctx, ID_STRING, s, 0);
		}
	default:
		abort();
	}
}

static int match(token * t, int *o)
{
	if (t == 0 || t->id != ID_EXPR2)
		return 0;
	if (t->left || t->right)
		return 0;
	if (*o == 0)
		return 1;
	while (*o)
		if (*o++ == t->u.number)
			return 1;
	return 0;
}

static void simplify(CTX * ctx, int o[], token ** head)
{
	token *p, *father, *left, *right;
	(void)ctx;

	if (*head == 0)
		return;
		
	if (match((*head)->next, o)) {
		p = *head;
		father = *head = p->next;
		left = p;
		right = father->next;
		if (father->left || father->right)
			abort();	
		father->next = right->next;
		right->next = left->next = 0;
		if (*o == 0) {
			father->right = left;
			father->left = right;
		} else {
			father->right = right;
			father->left = left;
		}
		simplify(ctx, o, head);
		return;
	}
	
	for (p = *head; p && p->next; p = p->next)
		if (match(p->next->next, o))
			break;
			
	if (p == 0 || p->next == 0)
		return;

	left = p->next;
	father = left->next;
	right = father->next;
	
	if (father->left || father->right)
		abort();

	p->next = father;
	
	father->next = right->next;
	left->next = right->next = 0;
	if (*o == 0) {
		father->left = right;
		father->right = left;
	} else {
		father->left = left;
		father->right = right;
	}

	simplify(ctx, o, head);
	return;
}

static token *simplify_a_bit(CTX * ctx, token * head)
{
	int i;
	token *t, *nh, *tmp;
	
	int ops[][6] = {
		{'*', '%', '/', 0, 0, 0},
		{'+', '-', 0, 0, 0, 0},
		{'z', 'x', 0, 0, 0, 0},
		{'<', '>', 'M', 'L', '@', 0},
		{'E', 'N', 0, 0, 0, 0},
		{'&', 0, 0, 0, 0, 0},
		{'^', 0, 0, 0, 0, 0},
		{'|', 0, 0, 0, 0, 0},
		{'a', 0, 0, 0, 0, 0},
		{'o', 0, 0, 0, 0, 0},
		{0, 0, 0, 0, 0, 0}
	};
	for (i = 0; i < 10; i++)
		simplify(ctx, ops[i], &head);
	/* relink it the other way (from the end) */
	nh = 0;
	for (t = head; t; t = tmp) {
		tmp = t->next;
		t->next = nh;
		nh = t;
	}
	
	/* simplify out all remaining opertars (should be only (X)='s) */
	simplify(ctx, ops[10], &nh);
	
	if (nh->next)
		abort();
		
	return nh;
}

static token *do_regexp(CTX *ctx)
{
	token *r;
	char *s;
	const char *e;

	s = get_regexp(ctx);
	if (s == 0) {
		poops(ctx, "unexpected EOF looking for regexp");
		return 0;
	}
	
	r = make_token(ctx, ID_REGEXP, 0, 0);
	e = regexp_make(s, r);
	xfree(s);

	if (e) {
		poops(ctx, "RE error: %s", e);
		kill_token(r);
		r = 0;
	}
	
	return r;
}

token *get_expression(CTX * ctx)
{
	token *head = 0, *p = 0;

	for (;;) {
		get_token(ctx);
		
		if (head == 0 && ctx->type == T_OP 
		    && ctx->iv && strchr(");", ctx->iv))
			return make_token(ctx, ID_INT, 0, 1); 
			/* return bogus true value */	
		else	
			ctx->leave_it = 1;
			
		if (link_end(&head, &p, get_val(ctx)) == 0)
			goto shit;

	re_hack:
		get_token(ctx);
		
		if (ctx->type == T_OP && ctx->iv && strchr(");,]", ctx->iv))
			break;
			
		if (ctx->type != T_OP) {
			poops(ctx, "hadn't found operator while scaning "
			      "expression");
			goto shit;
		}
		if (ctx->iv == 0 ||
		    strchr("@*-+=%^|&/<>ELMNzxao", ctx->iv & 0xff) == 0) {
			poops(ctx, "bad binary operator (%s) while scaning "
			      "expression", op2_name(ctx->iv));
			goto shit;
		}
		
		if (ctx->iv == '=')
			ctx->iv = 0x100;
	
		link_end(&head, &p, make_token(ctx, ID_EXPR2, 0, ctx->iv));

		if (ctx->iv == '@') {
			if (link_end(&head, &p, do_regexp(ctx)) == 0)
				goto shit;
			goto re_hack;
		}
	}

	return simplify_a_bit(ctx, head);

      shit:
	kill_token(head);
	return 0;
}
