/* include gnu.gpl, (*) 2000 Micha 'malekith' Moskal <malekith@topnet.pl>. 
 * $Id: script.c,v 1.18 2000/08/26 01:31:12 malekith Exp $ */

#include "vm.h"
#include "parser.h"
#include "proc.h"

static char *input_files[128];

void kill_token(token * t)
{
	if (t == 0)
		return;
		
	kill_token(t->left);
	kill_token(t->next);
	kill_token(t->right);
	kill_token(t->u.top);
	xfree(t->u.name);

	xfree(t);
}

char *input_file_name(vm_token *t)
{
	return t ? input_files[t->file_idx] : "NO FILE";
}

void prtf(const char *win, const char *fmt, ... );

void croak(vm_token * t, const char *fmt, ...)
{
	char *s1;
	va_list ap;

	va_start(ap, fmt);
	s1 = vxf(fmt, ap);
	va_end(ap);
	prtf(0, "!!! script error: %s:%d: %s", input_file_name(t),
	       t ? t->line : 0, s1);
	xfree(s1);
}

byte *parse(struct parser_state *);

static int add_file(char *fn)
{
	int i;
	static int last;
	
	if (input_files[0] == 0) {
		input_files[0] = xstrdup("NONE");
	}
	
	if (fn == 0)
		return 0;
		
	for (i = last + 1; i != last; i = (i + 1) & 0x7f)
		if (input_files[i] == 0)
			break;
	if (i == last)
		i = last + 1;
	if (i == 0)
		i++;
	xfree(input_files[last = i]);
	input_files[last] = fn;
	
	return last;
}

/* note that this is the only global var (unique for any given script
 * since there are also global cprocs and input_files,
 * but more then one script can share them w/o problems) used by entire
 * engine and it is used only in next three functions and iv() & sv(),
 * which are implemented only for out side apps and are not 
 * used inside scripting enigne */
frame global;

void init_scripting_engine()
{
	static int done;
	
	if (done)
		return;
	
	done = 1;
	
	add_file(0);
	init_cprocs();

	/* init global frame */
	setup_frame(&global, 0);
	global.procs = hash_new(0);
	global.procs->free_entry = free_proc;
	global.global = &global; /* ;) */
}

var *source_file(const char *fn, frame *f)
{
	struct parser_state ctx;
	byte *code;
	var *v;
	
	memset(&ctx,0,sizeof(ctx));
	ctx.p = ctx.line;
	ctx.fn = xstrdup(fn);
	ctx.file_idx = add_file((char*)ctx.fn);
	
	ctx.f = fopen(fn, "r");
	if (ctx.f == 0) {
		croak(0,"cannot open '%s'",fn);
		return 0;
	}
	
	ctx.frm = f->global;
	
	code = parse(&ctx);
	fclose(ctx.f);
	
	v = exec_loop(code, f->global);
	free_bc(code);
	
	return v;
}

void load_file(const char *fn)
{
	free_var(source_file(fn, &global));
}

var *veval(char *input, frame *f)
{
	struct parser_state ctx;
	var *v;
	byte *code;
	
	memset(&ctx, 0, sizeof(ctx));
	ctx.p = input;
	ctx.fn = "veval";

	ctx.frm = f->global;

	add_file(0);
	code = parse(&ctx);
	
	if (code == 0)
		return 0;
		
	v = exec_loop(code, f->global);
	free_bc(code);

	return v;
}

void eval(const char *input)
{
	char *p;

	p = xstrdup(input);
	free_var(veval(p, &global));
	xfree(p);
}

var *hsv(const char *h, const char *v)
{
	return fhv(&global, h, v, V_STRING);
}

var *hv(const char *h, const char *v)
{
	return fhv(&global, h, v, V_INT);
}

char *id_name(int id)
{
	char *names[] = {
		"error", "int", "string", "var", "proc", "cproc", "call",
		"def_int", "def_string", "def_ptr", "def_ihash", 
		"def_phash", "def_shash", "if", "while", "break", 
		"do_while", "continue", "return", "walk", "expr",
		"expr", "hash", "regexp", "last!"
	};
	if (id < 0)
		id = ID_LAST;
	if (id > ID_LAST)
		id = ID_LAST;
	return names[id];
}
