/* include gnu.gpl, (*) 2000 Micha 'malekith' Moskal <malekith@topnet.pl>. 
 * $Id: window.cc,v 1.6 2000/08/27 18:32:51 malekith Exp $ */

#include "config.h"
#include "sniffy.h"

#include <string.h>
#include <stdio.h>

IrcWindow *win_head, *win_topmost, *win_status;
// for alt+1 [1], alt+2 ... alt+shift+0 [10], alt+shift+1 [11] 
IrcWindow *win20[20]; 
void draw_statusline();

int IrcWindow::scroll_jump()
{
	int n = scroll_bottom - scroll_top;
	if (get_bool(B_HALF_SCROLL))
		n /= 2;
	return n;
}

void IrcWindow::pgup()
{
	if (offset == -1)
		offset = buffer.len() - (scroll_bottom - scroll_top);
	if (offset < 0) { offset = -1; return; }
	if ((offset -= scroll_jump()) < 0) 
		offset = 0; 
	draw();
}

void IrcWindow::pgdn()
{
	if (offset != -1 &&
	    (offset += scroll_jump()) >=
	    buffer.len() - (scroll_bottom - scroll_top)) 
	    		offset = -1;
	draw();
}

void put_line(int x, int y, const char *p)
{
	int color = 0x07;
	int no_colors;

	no_colors = get_bool(B_NO_COLORS);
	
	set_color(color);

	go(x, y);
	
	while (x < s_width && *p) {
		int c = *p;
		c &= 0xff;
		if (c == 27) 
			color = *++p;
		else {
			if (c == 30 || c == 9)
				c = ' ';	// nbsp or tab
			// we don't want to display what we can't read ;)
			if ((color & 0xf) == (color >> 4))
				color = 0x07;
			if (c < 32)
				print(c ^ 0100,
				      no_colors ? 
				      	0x70 : 
					((color << 4) | (color >> 4)) & 0xff);
			else
				print(c, no_colors ? 0x07 : color);
			x++;
		}
		p++;
	}
	while (x++ < s_width)
		print(' ', color);
}

void IrcWindow::draw_line(int n)
{
	int k;
	if (offset == -1)
		k = buffer.len() - ((scroll_bottom - scroll_top) - n + 1);
	else
		k = offset + n;
	char *z;
	if (k < 0) z = "";
	else if (k > buffer.len() - 1) z = "";
	else z = buffer[k];
	put_line(0,n,z);
}

void IrcWindow::draw_lines(int n)
{
	for (int i=scroll_bottom - n; i <= scroll_bottom ; i++)
		draw_line(i);
	changed = 0;
}

void IrcWindow::draw()
{
	if (win_topmost != this)
		return;
	draw_lines(scroll_bottom-scroll_top);
}

void IrcWindow::add(const char *str)
{
	int oc;

	oc = changed;
	
	buffer.w = s_width - 1;

	if (*channel || !get_bool(B_COPY_STATUS))
		changed = 1;

	int n = buffer.len();
	buffer.add(str);
	n = buffer.len() - n;
	
	if (offset == -1 && win_topmost == this
	    && n < scroll_bottom - scroll_top - 3 && !menu_active()) {
		scroll_up(n);
		draw_lines(n);
	} else {
		if (offset != -1 && get_bool(B_SCROLL_ON_PRINT))
			offset = -1;
		if (!menu_active())
			draw();
	}
	if (changed && !oc)
		draw_statusline();
}

void send_out(const char *ch, const char *msg)
{
	IrcWindow *w = 0;

	if (win_status == 0) {
		printf("[%s] %s\n",ch,msg);
		return;
	}

	if (ch && *ch)
		w = find_win(ch);
	if (w == 0)
		w = win_status;

	w->add(msg);
	if (w == win_status && win_topmost != win_status 
	    && get_bool(B_COPY_STATUS))
		win_topmost->add(msg);
}

void select_win(IrcWindow *w)
{
	if (!w) return;
	win_topmost = w;
	win_topmost->draw();
	draw_statusline();
	if (get_bool(B_WINDOW_INPUT_LINE))
		input_line = &win_topmost->iline;
	else
		input_line = &win_status->iline;
}

void add_window(const char *chan)
{
	IrcWindow *w = new IrcWindow;
	strncpy(w->channel, chan, sizeof(w->channel)-1);
	w->channel[sizeof(w->channel)-1] = 0;
	
	if (win_head) {
		IrcWindow *p = win_head;
		while (p->next) p = p->next;
		p->next = w;
	} else {
		win_head = w;
	}
	
	int no = 1;
	while (no < 20 && win20[no]) no++;
	if (no < 20) {
		win20[no] = w;
		w->no = no;
	}
	
	if (!win_topmost)
		win_topmost = w;
	if (!win_status)
		win_status = w;
	select_win(w);
}

void bring_to_top(const char *ch)
{
	select_win(find_win(ch));
}

void clear_win(const char *win)
{
	IrcWindow *w = find_win(win);
	if (w)
		w->clr();
}

void kill_window(const char *ch)
{
	if (!*ch)
		return; // can't kill status
	IrcWindow *w;
	for (w = win_head; w; w=w->next)
		if (strcasecmp(w->channel,ch) == 0)
			break;
	if (!w)
		return;
	if (w == win_head) {
		win_head = win_head->next;
	} else {
		IrcWindow *p = win_head;
		while (p->next != w)
			p = p->next;
		p->next = w->next;
	}
	if (w == win_topmost)
		select_win(win_status);
	if (win20[w->no] == w)
		win20[w->no] = 0;
	delete w;
}

IrcWindow *find_win(const char *ch)
{
	IrcWindow *w;
	for (w = win_head; w; w=w->next)
		if (strcasecmp(w->channel,ch) == 0)
			break;
	return w;
}

